package com.vieaura.camera_plugin.vieaura_camera_plugin

import android.Manifest
import android.app.Activity
import android.content.Intent
import android.content.pm.PackageManager
import android.graphics.Bitmap
import android.net.Uri
import android.os.Bundle
import android.provider.MediaStore
import android.util.Log
import androidx.annotation.NonNull
import androidx.core.content.ContextCompat
import androidx.core.app.ActivityCompat
import java.io.ByteArrayOutputStream
import java.io.InputStream
import java.io.File
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.ArrayList
import android.database.Cursor
import java.util.Timer
import kotlin.concurrent.schedule


class RealwearActivity : Activity() {
    private val BASIC_CAMERA_REQUEST_CODE = 1889
    private val FILEPROVIDER_CAMERA_REQUEST_CODE = 1998
    private val EXTRA_RESULT = "data"
    private var photos: ArrayList<String> = arrayListOf()
    private lateinit var outputDirectory: File
    private var isMultiPhotos = true
    private var isVideoRecord = false;
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        isMultiPhotos = getIntent().getBooleanExtra("MultiPhoto", false)
        isVideoRecord = getIntent().getBooleanExtra("VideoRecord", false)
        outputDirectory = getOutputDirectory()
        if (checkPermissions()) {
            launchApplet()
        }
    }

    private fun getOutputDirectory(): File {
        val mediaDir = externalMediaDirs.firstOrNull().let {
            File(
                it,
                resources.getString(R.string.app_name)
            ).apply { mkdirs() }
        }
        return if (mediaDir != null && mediaDir.exists())
            mediaDir else filesDir
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, intent: Intent?) {
        if (resultCode == RESULT_OK && intent != null) {
            if (isVideoRecord == true) {
                Log.v("RealPath", Uri.parse(("content://media/" + intent.data?.path)!!).toString())

                val inputStream: InputStream? =
                    getContentResolver().openInputStream(Uri.parse(("content://media/" + intent.data?.path)!!))
                val videoFile = File(
                    outputDirectory,
                    SimpleDateFormat(
                        "yyyy-MM-dd-HH-mm-ss-SSS", Locale.US
                    ).format(System.currentTimeMillis()) + ".mp4"
                )
                videoFile.writeBytes((inputStream?.readBytes())!!);
                inputStream.close()

                VieauraCameraPlugin.onVideoResult(videoFile.path)
                finish()
            } else {
                // add the image path to list
                var bitmap: Bitmap? = intent.extras!!.getParcelable<Bitmap>(EXTRA_RESULT)
                val stream = ByteArrayOutputStream()
                bitmap?.compress(Bitmap.CompressFormat.PNG, 100, stream)
                val bytes = stream.toByteArray()

                val photoFile = File(
                    outputDirectory,
                    SimpleDateFormat(
                        "yyyy-MM-dd-HH-mm-ss-SSS", Locale.US
                    ).format(System.currentTimeMillis()) + ".png"
                )
                photoFile.writeBytes(bytes);
                stream.close()

                photos.add(photoFile.path)
                if (isMultiPhotos == true) {
                    if (checkPermissions()) {
                        launchApplet()
                    }
                } else {
                    VieauraCameraPlugin.onTakePhotosResults(photos)
                    finish()
                }
            }
        } else if (resultCode == RESULT_CANCELED) {
            if (isMultiPhotos == true) {
                VieauraCameraPlugin.onTakePhotosResults(photos)
                finish()
            }
        } else {
            VieauraCameraPlugin.onEmptyResult()
        }
    }

    override fun onRequestPermissionsResult(
        requestCode: Int, permissions: Array<String>,
        grantResults: IntArray
    ) {
        when (requestCode) {
            BASIC_CAMERA_REQUEST_CODE -> {
                if ((grantResults.isNotEmpty() && grantResults[0] == PackageManager.PERMISSION_GRANTED)) {
                    launchApplet()
                } else {
                    VieauraCameraPlugin.onEmptyResult()
                    finish()
                }
                return
            }
            else -> {
            }
        }
    }

    private fun checkPermissions(): Boolean {
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.CAMERA) !=
            PackageManager.PERMISSION_GRANTED
        ) {

            ActivityCompat.requestPermissions(
                this,
                arrayOf(Manifest.permission.CAMERA),
                BASIC_CAMERA_REQUEST_CODE
            )

            return false
        }

        return true
    }

    private fun launchApplet() {
        if (isVideoRecord == true) {
            val intent = Intent(MediaStore.ACTION_VIDEO_CAPTURE)
            startActivityForResult(intent, FILEPROVIDER_CAMERA_REQUEST_CODE)
        } else {
            val intent = Intent(MediaStore.ACTION_IMAGE_CAPTURE)

//            val dir = File(
//                Environment.getExternalStoragePublicDirectory(
//                    Environment.DIRECTORY_DCIM
//                ), "Camera"
//            )
//            val file = File(outputDirectory, "vapps-" + UUID.randomUUID().toString() + ".jpg")
//
//            val contentUri = Uri.fromFile(file)
//
//            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
//            intent.addFlags(Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
//            intent.putExtra(MediaStore.EXTRA_OUTPUT, contentUri);

            startActivityForResult(intent, BASIC_CAMERA_REQUEST_CODE)
        }
    }
}